@@ -1,6 +1,7 @@
+import logfire
 from typing import List, Tuple, Optional
 from sqlalchemy.orm import Session
-from sqlalchemy import func, text
+from sqlalchemy import func, text, BigInteger
 from .schemas import (
     Library,
     NewLibraryParam,
@@ -24,14 +25,12 @@ from .models import (
     TagModel,
     EntityMetadataModel,
     EntityTagModel,
+    EntityPluginStatusModel,
 )
-import numpy as np
-from collections import defaultdict
-from .embedding import get_embeddings
 import logging
-from sqlite_vec import serialize_float32
-import time
-import asyncio
+from sqlalchemy.sql import text
+from sqlalchemy.orm import joinedload
+from sqlalchemy.sql import or_
 
 logger = logging.getLogger(__name__)
 
@@ -73,7 +72,7 @@ def create_library(library: NewLibraryParam, db: Session) -> Library:
 
 
 def get_libraries(db: Session) -> List[Library]:
-    return db.query(LibraryModel).all()
+    return db.query(LibraryModel).order_by(LibraryModel.id.asc()).all()
 
 
 def get_library_by_name(library_name: str, db: Session) -> Library | None:
@@ -97,10 +96,14 @@ def add_folders(library_id: int, folders: NewFoldersParam, db: Session) -> Libra
         db.refresh(db_folder)
 
     db_library = db.query(LibraryModel).filter(LibraryModel.id == library_id).first()
-    return Library(**db_library.__dict__)
+    return Library.model_validate(db_library, from_attributes=True)
 
 
-def create_entity(library_id: int, entity: NewEntityParam, db: Session) -> Entity:
+def create_entity(
+    library_id: int,
+    entity: NewEntityParam,
+    db: Session,
+) -> Entity:
     tags = entity.tags
     metadata_entries = entity.metadata_entries
 
@@ -145,30 +148,147 @@ def create_entity(library_id: int, entity: NewEntityParam, db: Session) -> Entit
             )
             db.add(entity_metadata)
     db.commit()
-    db.refresh(db_entity)
-    return Entity(**db_entity.__dict__)
+    
+    return get_entity_by_id(db_entity.id, db, include_relationships=True)
 
 
-def get_entity_by_id(entity_id: int, db: Session) -> Entity | None:
-    return db.query(EntityModel).filter(EntityModel.id == entity_id).first()
+def _load_entity_with_relationships(db: Session, entity_id: int) -> EntityModel:
+    """Helper function to load entity with all relationships"""
+    return (
+        db.query(EntityModel)
+        .options(
+            joinedload(EntityModel.metadata_entries),
+            joinedload(EntityModel.tags),
+            joinedload(EntityModel.plugin_status),
+        )
+        .filter(EntityModel.id == entity_id)
+        .first()
+    )
+
+
+def get_entity_by_id(entity_id: int, db: Session, include_relationships: bool = False) -> Entity | None:
+    """Get entity by ID with optional relationships"""
+    if include_relationships:
+        db_entity = _load_entity_with_relationships(db, entity_id)
+    else:
+        db_entity = db.query(EntityModel).filter(EntityModel.id == entity_id).first()
+    
+    if db_entity is None:
+        return None
+        
+    return Entity.model_validate(db_entity, from_attributes=True)
 
 
 def get_entities_of_folder(
-    library_id: int, folder_id: int, db: Session, limit: int = 10, offset: int = 0
+    library_id: int,
+    folder_id: int,
+    db: Session,
+    limit: int = 10,
+    offset: int = 0,
+    path_prefix: str | None = None,
+    unprocessed_only: bool = False,
+    order_by: str = "last_scan_at:desc",
 ) -> Tuple[List[Entity], int]:
-    folder = (
-        db.query(FolderModel)
-        .filter(FolderModel.id == folder_id, FolderModel.library_id == library_id)
-        .first()
+    # Define allowed columns for ordering
+    ALLOWED_ORDER_COLUMNS = {
+        "last_scan_at",
+        "file_created_at",
+        "file_last_modified_at",
+        "filename",
+        "filepath",
+        "id",
+    }
+
+    # First get the entity IDs with limit and offset
+    base_query = db.query(EntityModel.id).filter(
+        EntityModel.folder_id == folder_id,
+        EntityModel.library_id == library_id,
     )
-    if folder is None:
-        return [], 0
 
-    query = db.query(EntityModel).filter(EntityModel.folder_id == folder_id)
+    # Add path_prefix filter if provided
+    if path_prefix:
+        base_query = base_query.filter(EntityModel.filepath.like(f"{path_prefix}%"))
 
-    total_count = query.count()
+    # Add unprocessed filter if requested
+    if unprocessed_only:
+        # Get all plugins for the library
+        library_plugins = (
+            db.query(PluginModel.id)
+            .join(LibraryPluginModel)
+            .filter(LibraryPluginModel.library_id == library_id)
+            .all()
+        )
+        library_plugin_ids = [p.id for p in library_plugins]
 
-    entities = query.limit(limit).offset(offset).all()
+        if library_plugin_ids:
+            # Count plugins that have processed each entity
+            plugin_count_subquery = (
+                db.query(
+                    EntityPluginStatusModel.entity_id,
+                    func.count(EntityPluginStatusModel.plugin_id).label("plugin_count"),
+                )
+                .filter(EntityPluginStatusModel.plugin_id.in_(library_plugin_ids))
+                .group_by(EntityPluginStatusModel.entity_id)
+                .subquery()
+            )
+
+            # Join with the plugin count subquery and filter entities that haven't been processed by all plugins
+            base_query = base_query.outerjoin(
+                plugin_count_subquery,
+                EntityModel.id == plugin_count_subquery.c.entity_id,
+            ).filter(
+                or_(
+                    plugin_count_subquery.c.plugin_count.is_(
+                        None
+                    ),  # No plugins have processed
+                    plugin_count_subquery.c.plugin_count
+                    < len(library_plugin_ids),  # Not all plugins have processed
+                )
+            )
+
+    # Determine the order by field and direction
+    try:
+        order_field, order_direction = order_by.split(":")
+    except ValueError:
+        raise ValueError(
+            f"Invalid order_by format: {order_by}. Expected format: 'column:direction' where direction is 'asc' or 'desc'"
+        )
+
+    if order_field not in ALLOWED_ORDER_COLUMNS:
+        raise ValueError(
+            f"Invalid order column: {order_field}. Allowed columns: {', '.join(sorted(ALLOWED_ORDER_COLUMNS))}"
+        )
+
+    if order_direction.lower() not in ["asc", "desc"]:
+        raise ValueError(
+            f"Invalid order direction: {order_direction}. Must be either 'asc' or 'desc'"
+        )
+
+    order_column = getattr(EntityModel, order_field)
+    order_column = (
+        order_column.desc() if order_direction.lower() == "desc" else order_column.asc()
+    )
+
+    # Order by the specified field and direction
+    id_query = base_query.order_by(order_column)
+
+    # Get total count and paginated results
+    total_count = id_query.count()
+    entity_ids = id_query.limit(limit).offset(offset).all()
+    entity_ids = [id[0] for id in entity_ids]
+
+    # Then get the full entities with relationships for those IDs
+    entities = (
+        db.query(EntityModel)
+        .options(
+            joinedload(EntityModel.metadata_entries),
+            joinedload(EntityModel.tags),
+            joinedload(EntityModel.plugin_status),
+        )
+        .filter(EntityModel.id.in_(entity_ids))
+        .order_by(order_column)  # Keep consistent with the ID query
+        .all()
+    )
 
     return entities, total_count
 
@@ -178,12 +298,28 @@ def get_entity_by_filepath(filepath: str, db: Session) -> Entity | None:
 
 
 def get_entities_by_filepaths(filepaths: List[str], db: Session) -> List[Entity]:
-    return db.query(EntityModel).filter(EntityModel.filepath.in_(filepaths)).all()
+    return (
+        db.query(EntityModel)
+        .options(
+            joinedload(EntityModel.metadata_entries),
+            joinedload(EntityModel.tags),
+            joinedload(EntityModel.plugin_status),
+        )
+        .filter(EntityModel.filepath.in_(filepaths))
+        .all()
+    )
 
 
 def remove_entity(entity_id: int, db: Session):
     entity = db.query(EntityModel).filter(EntityModel.id == entity_id).first()
     if entity:
+        # Delete the entity from FTS and vec tables first
+        db.execute(text("DELETE FROM entities_fts WHERE id = :id"), {"id": entity_id})
+        db.execute(
+            text("DELETE FROM entities_vec_v2 WHERE rowid = :id"), {"id": entity_id}
+        )
+
+        # Then delete the entity itself
         db.delete(entity)
         db.commit()
     else:
@@ -199,7 +335,7 @@ def create_plugin(newPlugin: NewPluginParam, db: Session) -> Plugin:
 
 
 def get_plugins(db: Session) -> List[Plugin]:
-    return db.query(PluginModel).all()
+    return db.query(PluginModel).order_by(PluginModel.id.asc()).all()
 
 
 def get_plugin_by_name(plugin_name: str, db: Session) -> Plugin | None:
@@ -217,20 +353,25 @@ def add_plugin_to_library(library_id: int, plugin_id: int, db: Session):
     db.refresh(library_plugin)
 
 
-def find_entity_by_id(entity_id: int, db: Session) -> Entity | None:
-    db_entity = db.query(EntityModel).filter(EntityModel.id == entity_id).first()
-    if db_entity is None:
-        return None
-    return Entity(**db_entity.__dict__)
-
-
 def find_entities_by_ids(entity_ids: List[int], db: Session) -> List[Entity]:
-    db_entities = db.query(EntityModel).filter(EntityModel.id.in_(entity_ids)).all()
-    return [Entity(**entity.__dict__) for entity in db_entities]
+    db_entities = (
+        db.query(EntityModel)
+        .options(
+            joinedload(EntityModel.metadata_entries),
+            joinedload(EntityModel.tags),
+            joinedload(EntityModel.plugin_status),
+        )
+        .filter(EntityModel.id.in_(entity_ids))
+        .all()
+    )
+    return [Entity.model_validate(entity, from_attributes=True) for entity in db_entities]
 
 
 def update_entity(
-    entity_id: int, updated_entity: UpdateEntityParam, db: Session
+    entity_id: int,
+    updated_entity: UpdateEntityParam,
+    db: Session,
+    force: bool = False,
 ) -> Entity:
     db_entity = db.query(EntityModel).filter(EntityModel.id == entity_id).first()
 
@@ -285,9 +426,16 @@ def update_entity(
             db.add(entity_metadata)
             db_entity.metadata_entries.append(entity_metadata)
 
+    # If force is True, clear all plugin_status entries
+    if force:
+        db.query(EntityPluginStatusModel).filter(
+            EntityPluginStatusModel.entity_id == entity_id
+        ).delete()
+        db.commit()
+
     db.commit()
-    db.refresh(db_entity)
-    return Entity(**db_entity.__dict__)
+
+    return get_entity_by_id(entity_id, db, include_relationships=True)
 
 
 def touch_entity(entity_id: int, db: Session) -> bool:
@@ -301,14 +449,17 @@ def touch_entity(entity_id: int, db: Session) -> bool:
         return False
 
 
-def update_entity_tags(entity_id: int, tags: List[str], db: Session) -> Entity:
-    db_entity = get_entity_by_id(entity_id, db)
+def update_entity_tags(
+    entity_id: int,
+    tags: List[str],
+    db: Session,
+) -> Entity:
+    db_entity = db.query(EntityModel).filter(EntityModel.id == entity_id).first()
     if not db_entity:
         raise ValueError(f"Entity with id {entity_id} not found")
 
     # Clear existing tags
     db.query(EntityTagModel).filter(EntityTagModel.entity_id == entity_id).delete()
-    db.commit()
 
     for tag_name in tags:
         tag = db.query(TagModel).filter(TagModel.name == tag_name).first()
@@ -323,13 +474,17 @@ def update_entity_tags(entity_id: int, tags: List[str], db: Session) -> Entity:
             source=MetadataSource.PLUGIN_GENERATED,
         )
         db.add(entity_tag)
+
+    # Update last_scan_at in the same transaction
+    db_entity.last_scan_at = func.now()
+
     db.commit()
-    db.refresh(db_entity)
-    return Entity(**db_entity.__dict__)
+
+    return get_entity_by_id(entity_id, db, include_relationships=True)
 
 
 def add_new_tags(entity_id: int, tags: List[str], db: Session) -> Entity:
-    db_entity = get_entity_by_id(entity_id, db)
+    db_entity = db.query(EntityModel).filter(EntityModel.id == entity_id).first()
     if not db_entity:
         raise ValueError(f"Entity with id {entity_id} not found")
 
@@ -349,15 +504,21 @@ def add_new_tags(entity_id: int, tags: List[str], db: Session) -> Entity:
             source=MetadataSource.PLUGIN_GENERATED,
         )
         db.add(entity_tag)
+
+    # Update last_scan_at in the same transaction
+    db_entity.last_scan_at = func.now()
+
     db.commit()
-    db.refresh(db_entity)
-    return Entity(**db_entity.__dict__)
+
+    return get_entity_by_id(entity_id, db, include_relationships=True)
 
 
 def update_entity_metadata_entries(
-    entity_id: int, updated_metadata: List[EntityMetadataParam], db: Session
+    entity_id: int,
+    updated_metadata: List[EntityMetadataParam],
+    db: Session,
 ) -> Entity:
-    db_entity = get_entity_by_id(entity_id, db)
+    db_entity = db.query(EntityModel).filter(EntityModel.id == entity_id).first()
 
     existing_metadata_entries = (
         db.query(EntityMetadataModel)
@@ -398,9 +559,12 @@ def update_entity_metadata_entries(
             db.add(entity_metadata)
             db_entity.metadata_entries.append(entity_metadata)
 
+    # Update last_scan_at in the same transaction
+    db_entity.last_scan_at = func.now()
+
     db.commit()
-    db.refresh(db_entity)
-    return Entity(**db_entity.__dict__)
+
+    return get_entity_by_id(entity_id, db, include_relationships=True)
 
 
 def get_plugin_by_id(plugin_id: int, db: Session) -> Plugin | None:
@@ -424,175 +588,125 @@ def remove_plugin_from_library(library_id: int, plugin_id: int, db: Session):
         raise ValueError(f"Plugin {plugin_id} not found in library {library_id}")
 
 
-def and_words(input_string):
-    words = input_string.split()
-    result = " AND ".join(words)
-    return result
-
-
-def full_text_search(
-    query: str,
+def list_entities(
     db: Session,
     limit: int = 200,
     library_ids: Optional[List[int]] = None,
     start: Optional[int] = None,
     end: Optional[int] = None,
-) -> List[int]:
-    and_query = and_words(query)
-
-    sql_query = """
-    SELECT entities.id FROM entities
-    JOIN entities_fts ON entities.id = entities_fts.id
-    WHERE entities_fts MATCH jieba_query(:query)
-    AND entities.file_type_group = 'image'
-    """
-
-    params = {"query": and_query, "limit": limit}
-
-    if library_ids:
-        library_ids_str = ", ".join(f"'{id}'" for id in library_ids)
-        sql_query += f" AND entities.library_id IN ({library_ids_str})"
-    if start is not None and end is not None:
-        sql_query += " AND strftime('%s', entities.file_created_at, 'utc') BETWEEN :start AND :end"
-        params["start"] = str(start)
-        params["end"] = str(end)
-
-    sql_query += " ORDER BY bm25(entities_fts), entities.last_scan_at DESC LIMIT :limit"
-
-    result = db.execute(text(sql_query), params).fetchall()
-
-    logger.info(f"Full-text search sql: {sql_query}")
-    logger.info(f"Full-text search params: {params}")
-    ids = [row[0] for row in result]
-    logger.info(f"Full-text search results: {ids}")
-    return ids
-
-
-async def vec_search(
-    query: str,
-    db: Session,
-    limit: int = 200,
-    library_ids: Optional[List[int]] = None,
-    start: Optional[int] = None,
-    end: Optional[int] = None,
-) -> List[int]:
-    query_embedding = await get_embeddings([query])
-    if not query_embedding:
-        return []
-
-    query_embedding = query_embedding[0]
-
-    sql_query = """
-    SELECT entities.id FROM entities
-    JOIN entities_vec ON entities.id = entities_vec.rowid
-    WHERE entities_vec.embedding MATCH :embedding
-    AND entities.file_type_group = 'image'
-    """
-
-    params = {"embedding": serialize_float32(query_embedding), "limit": limit}
-
-    if library_ids:
-        library_ids_str = ", ".join(f"'{id}'" for id in library_ids)
-        sql_query += f" AND entities.library_id IN ({library_ids_str})"
-
-    if start is not None and end is not None:
-        sql_query += (
-            " AND strftime('%s', entities.file_created_at, 'utc') BETWEEN :start AND :end"
+) -> List[Entity]:
+    query = (
+        db.query(EntityModel)
+        .options(
+            joinedload(EntityModel.metadata_entries),
+            joinedload(EntityModel.tags),
+            joinedload(EntityModel.plugin_status),
         )
-        params["start"] = str(start)
-        params["end"] = str(end)
-
-    sql_query += " AND K = :limit ORDER BY distance, entities.last_scan_at DESC"
-
-    result = db.execute(text(sql_query), params).fetchall()
-
-    ids = [row[0] for row in result]
-    logger.info(f"Vector search results: {ids}")
-    return ids
-
-
-def reciprocal_rank_fusion(
-    fts_results: List[int], vec_results: List[int], k: int = 60
-) -> List[Tuple[int, float]]:
-    rank_dict = defaultdict(float)
-
-    for rank, result_id in enumerate(fts_results):
-        rank_dict[result_id] += 1 / (k + rank + 1)
-
-    for rank, result_id in enumerate(vec_results):
-        rank_dict[result_id] += 1 / (k + rank + 1)
-
-    sorted_results = sorted(rank_dict.items(), key=lambda x: x[1], reverse=True)
-    return sorted_results
-
-
-async def hybrid_search(
-    query: str,
-    db: Session,
-    limit: int = 200,
-    library_ids: Optional[List[int]] = None,
-    start: Optional[int] = None,
-    end: Optional[int] = None,
-) -> List[Entity]:
-    start_time = time.time()
-
-    fts_start = time.time()
-    fts_results = full_text_search(query, db, limit, library_ids, start, end)
-    fts_end = time.time()
-    logger.info(f"Full-text search took {fts_end - fts_start:.4f} seconds")
-
-    vec_start = time.time()
-    vec_results = await vec_search(query, db, limit, library_ids, start, end)
-    vec_end = time.time()
-    logger.info(f"Vector search took {vec_end - vec_start:.4f} seconds")
-
-    fusion_start = time.time()
-    combined_results = reciprocal_rank_fusion(fts_results, vec_results)
-    fusion_end = time.time()
-    logger.info(f"Reciprocal rank fusion took {fusion_end - fusion_start:.4f} seconds")
-
-    sorted_ids = [id for id, _ in combined_results][:limit]
-    logger.info(f"Hybrid search results (sorted IDs): {sorted_ids}")
-
-    entities_start = time.time()
-    entities = find_entities_by_ids(sorted_ids, db)
-    entities_end = time.time()
-    logger.info(
-        f"Finding entities by IDs took {entities_end - entities_start:.4f} seconds"
+        .filter(EntityModel.file_type_group == "image")
     )
 
-    # Create a dictionary mapping entity IDs to entities
-    entity_dict = {entity.id: entity for entity in entities}
-
-    # Return entities in the order of sorted_ids
-    result = [entity_dict[id] for id in sorted_ids if id in entity_dict]
-
-    end_time = time.time()
-    total_time = end_time - start_time
-    logger.info(f"Total hybrid search time: {total_time:.4f} seconds")
-
-    return result
-
-
-async def list_entities(
-    db: Session,
-    limit: int = 200,
-    library_ids: Optional[List[int]] = None,
-    start: Optional[int] = None,
-    end: Optional[int] = None,
-) -> List[Entity]:
-    query = db.query(EntityModel).filter(EntityModel.file_type_group == "image")
-
     if library_ids:
         query = query.filter(EntityModel.library_id.in_(library_ids))
 
     if start is not None and end is not None:
+        # Convert timestamp to Unix timestamp using EXTRACT(EPOCH FROM timestamp)
+        # This works in both PostgreSQL and SQLite (SQLite will use its own implementation)
         query = query.filter(
-            func.strftime("%s", EntityModel.file_created_at, 'utc').between(
-                str(start), str(end)
-            )
+            func.extract("epoch", EntityModel.file_created_at)
+            .cast(BigInteger)
+            .between(start, end)
         )
 
-    entities = query.order_by(EntityModel.last_scan_at.desc()).limit(limit).all()
+    entities = query.order_by(EntityModel.file_created_at.desc()).limit(limit).all()
 
-    return [Entity(**entity.__dict__) for entity in entities]
+    return [Entity.model_validate(entity, from_attributes=True) for entity in entities]
+
+
+def get_entity_context(
+    db: Session, library_id: int, entity_id: int, prev: int = 0, next: int = 0
+) -> Tuple[List[Entity], List[Entity]]:
+    """
+    Get the context (previous and next entities) for a given entity.
+    Returns a tuple of (previous_entities, next_entities).
+    """
+    # First get the target entity to get its timestamp
+    target_entity = (
+        db.query(EntityModel)
+        .options(
+            joinedload(EntityModel.metadata_entries),
+            joinedload(EntityModel.tags),
+            joinedload(EntityModel.plugin_status),
+        )
+        .filter(
+            EntityModel.id == entity_id,
+            EntityModel.library_id == library_id,
+        )
+        .first()
+    )
+
+    if not target_entity:
+        return [], []
+
+    # Get previous entities
+    prev_entities = []
+    if prev > 0:
+        prev_entities = (
+            db.query(EntityModel)
+            .filter(
+                EntityModel.library_id == library_id,
+                EntityModel.file_created_at < target_entity.file_created_at,
+            )
+            .order_by(EntityModel.file_created_at.desc())
+            .limit(prev)
+            .all()
+        )
+        # Reverse the list to get chronological order and convert to Entity models
+        prev_entities = [Entity.model_validate(entity, from_attributes=True) for entity in prev_entities][::-1]
+
+    # Get next entities
+    next_entities = []
+    if next > 0:
+        next_entities = (
+            db.query(EntityModel)
+            .filter(
+                EntityModel.library_id == library_id,
+                EntityModel.file_created_at > target_entity.file_created_at,
+            )
+            .order_by(EntityModel.file_created_at.asc())
+            .limit(next)
+            .all()
+        )
+        # Convert to Entity models
+        next_entities = [Entity.model_validate(entity, from_attributes=True) for entity in next_entities]
+
+    return prev_entities, next_entities
+
+
+def record_plugin_processed(entity_id: int, plugin_id: int, db: Session):
+    """Record that an entity has been processed by a plugin"""
+    status = EntityPluginStatusModel(entity_id=entity_id, plugin_id=plugin_id)
+    db.merge(status)  # merge will insert or update
+    db.commit()
+
+
+def get_pending_plugins(entity_id: int, library_id: int, db: Session) -> List[int]:
+    """Get list of plugin IDs that haven't processed this entity yet"""
+    # Get all plugins associated with the library
+    library_plugins = (
+        db.query(PluginModel.id)
+        .join(LibraryPluginModel)
+        .filter(LibraryPluginModel.library_id == library_id)
+        .all()
+    )
+    library_plugin_ids = [p.id for p in library_plugins]
+
+    # Get plugins that have already processed this entity
+    processed_plugins = (
+        db.query(EntityPluginStatusModel.plugin_id)
+        .filter(EntityPluginStatusModel.entity_id == entity_id)
+        .all()
+    )
+    processed_plugin_ids = [p.plugin_id for p in processed_plugins]
+
+    # Return plugins that need to process this entity
+    return list(set(library_plugin_ids) - set(processed_plugin_ids))
