@@ -1,9 +1,11 @@
 import json
 import os
 import pytest
+from datetime import datetime
+from copy import deepcopy
 
 from fastapi.testclient import TestClient
-from sqlalchemy import create_engine
+from sqlalchemy import create_engine, event, text
 from sqlalchemy.orm import sessionmaker
 from sqlalchemy.pool import StaticPool
 from pathlib import Path
@@ -16,62 +18,39 @@ from memos.schemas import (
     NewEntityParam,
     UpdateEntityParam,
     NewFoldersParam,
+    NewFolderParam,
     EntityMetadataParam,
     MetadataType,
     UpdateEntityTagsParam,
     UpdateEntityMetadataParam,
+    FolderType,
 )
 from memos.models import Base
+from memos.databases.initializers import SQLiteInitializer
+from memos.config import settings
+from memos.search import create_search_provider
 
 
-engine = create_engine(
-    "sqlite:///:memory:",
+# Create a test settings object by copying the original settings
+test_settings = deepcopy(settings)
+test_settings.database_path = "sqlite:///:memory:"
+
+# Use SQLite for testing by default
+test_engine = create_engine(
+    test_settings.database_url,
     connect_args={"check_same_thread": False},
     poolclass=StaticPool,
 )
-TestingSessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)
 
+# Initialize SQLite with the test engine
+test_initializer = SQLiteInitializer(test_engine, test_settings)
+test_initializer.init_extensions()
 
-def load_fixture(filename):
-    with open(Path(__file__).parent / "fixtures" / filename, "r") as file:
-        return json.load(file)
+# Create and override the search provider with test database URL
+test_search_provider = create_search_provider(test_settings.database_url)
+app.state.search_provider = test_search_provider  # Store in app state
 
-
-def setup_library_with_entity(client):
-    # Create a new library
-    new_library = NewLibraryParam(name="Test Library for Metadata")
-    library_response = client.post(
-        "/libraries", json=new_library.model_dump(mode="json")
-    )
-    assert library_response.status_code == 200
-    library_id = library_response.json()["id"]
-
-    # Create a new folder in the library
-    new_folder = NewFoldersParam(folders=["/tmp"])
-    folder_response = client.post(
-        f"/libraries/{library_id}/folders", json=new_folder.model_dump(mode="json")
-    )
-    assert folder_response.status_code == 200
-    folder_id = folder_response.json()["folders"][0]["id"]
-
-    # Create a new entity in the folder
-    new_entity = NewEntityParam(
-        filename="metadata_test_file.txt",
-        filepath="/tmp/metadata_folder/metadata_test_file.txt",
-        size=5678,
-        file_created_at="2023-01-01T00:00:00",
-        file_last_modified_at="2023-01-01T00:00:00",
-        file_type="txt",
-        file_type_group="text",
-        folder_id=folder_id,
-    )
-    entity_response = client.post(
-        f"/libraries/{library_id}/entities", json=new_entity.model_dump(mode="json")
-    )
-    assert entity_response.status_code == 200
-    entity_id = entity_response.json()["id"]
-
-    return library_id, folder_id, entity_id
+TestingSessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=test_engine)
 
 
 def override_get_db():
@@ -88,17 +67,80 @@ app.dependency_overrides[get_db] = override_get_db
 # Setup a fixture for the FastAPI test client
 @pytest.fixture
 def client():
-    Base.metadata.create_all(bind=engine)
+    # Create all base tables
+    Base.metadata.create_all(bind=test_engine)
+
+    # Create FTS and Vec tables for SQLite
+    test_initializer.init_specific_features()
+
     with TestClient(app) as client:
         yield client
-    Base.metadata.drop_all(bind=engine)
+
+    # Clean up database
+    Base.metadata.drop_all(bind=test_engine)
+    with test_engine.connect() as conn:
+        conn.execute(text("DROP TABLE IF EXISTS entities_fts"))
+        conn.execute(text("DROP TABLE IF EXISTS entities_vec_v2"))
+        conn.commit()
+
+
+def load_fixture(filename):
+    with open(Path(__file__).parent / "fixtures" / filename, "r") as file:
+        return json.load(file)
+
+
+def setup_library_with_entity(client):
+    # Create a new library
+    new_library = NewLibraryParam(name="Test Library for Metadata")
+    library_response = client.post(
+        "/api/libraries", json=new_library.model_dump(mode="json")
+    )
+    assert library_response.status_code == 200
+    library_id = library_response.json()["id"]
+
+    # Create a new folder in the library
+    new_folder = NewFoldersParam(
+        folders=[
+            NewFolderParam(
+                path="/tmp", last_modified_at=datetime.now(), type=FolderType.DEFAULT
+            )
+        ]
+    )
+    folder_response = client.post(
+        f"/api/libraries/{library_id}/folders", json=new_folder.model_dump(mode="json")
+    )
+    assert folder_response.status_code == 200
+    folder_id = folder_response.json()["folders"][0]["id"]
+
+    # Create a new entity in the folder
+    new_entity = NewEntityParam(
+        filename="metadata_test_file.txt",
+        filepath="/tmp/metadata_folder/metadata_test_file.txt",
+        size=5678,
+        file_created_at="2023-01-01T00:00:00",
+        file_last_modified_at="2023-01-01T00:00:00",
+        file_type="txt",
+        file_type_group="text",
+        folder_id=folder_id,
+    )
+    entity_response = client.post(
+        f"/api/libraries/{library_id}/entities", json=new_entity.model_dump(mode="json")
+    )
+    assert entity_response.status_code == 200
+    entity_id = entity_response.json()["id"]
+
+    # Update the entity's index
+    index_response = client.post(f"/api/entities/{entity_id}/index")
+    assert index_response.status_code == 204
+
+    return library_id, folder_id, entity_id
 
 
 # Test the new_library endpoint
 def test_new_library(client):
     library_param = NewLibraryParam(name="Test Library")
     # Make a POST request to the /libraries endpoint
-    response = client.post("/libraries", json=library_param.model_dump())
+    response = client.post("/api/libraries", json=library_param.model_dump())
     # Check that the response is successful
     assert response.status_code == 200
     # Check the response data
@@ -110,7 +152,7 @@ def test_new_library(client):
     }
 
     # Test for duplicate library name
-    duplicate_response = client.post("/libraries", json=library_param.model_dump())
+    duplicate_response = client.post("/api/libraries", json=library_param.model_dump())
     # Check that the response indicates a failure due to duplicate name
     assert duplicate_response.status_code == 400
     assert duplicate_response.json() == {
@@ -119,33 +161,59 @@ def test_new_library(client):
 
 
 def test_list_libraries(client):
-    # Setup data: Create a new library
-    new_library = NewLibraryParam(name="Sample Library", folders=["/tmp"])
-    client.post("/libraries", json=new_library.model_dump(mode="json"))
+    # Setup data: Create a new library with a folder
+    new_library = NewLibraryParam(
+        name="Sample Library",
+        folders=[
+            NewFolderParam(
+                path="/tmp", last_modified_at=datetime.now(), type=FolderType.DEFAULT
+            )
+        ],
+    )
+    client.post("/api/libraries", json=new_library.model_dump(mode="json"))
 
     # Make a GET request to the /libraries endpoint
-    response = client.get("/libraries")
+    response = client.get("/api/libraries")
 
     # Check that the response is successful
     assert response.status_code == 200
 
     # Check the response data
+    response_data = response.json()
+    for folder in response_data[0]["folders"]:
+        assert "last_modified_at" in folder
+        assert isinstance(folder["last_modified_at"], str)
+        del folder["last_modified_at"]
+
     expected_data = [
         {
             "id": 1,
             "name": "Sample Library",
-            "folders": [{"id": 1, "path": "/tmp"}],
+            "folders": [
+                {
+                    "id": 1,
+                    "path": "/tmp",
+                    "type": "DEFAULT",
+                }
+            ],
             "plugins": [],
         }
     ]
-    assert response.json() == expected_data
+    assert response_data == expected_data
 
 
 def test_new_entity(client):
     # Setup data: Create a new library
-    new_library = NewLibraryParam(name="Library for Entity Test", folders=["/tmp"])
+    new_library = NewLibraryParam(
+        name="Library for Entity Test",
+        folders=[
+            NewFolderParam(
+                path="/tmp", last_modified_at=datetime.now(), type=FolderType.DEFAULT
+            )
+        ],
+    )
     library_response = client.post(
-        "/libraries", json=new_library.model_dump(mode="json")
+        "/api/libraries", json=new_library.model_dump(mode="json")
     )
     library_id = library_response.json()["id"]
     folder_id = library_response.json()["folders"][0]["id"]
@@ -162,7 +230,7 @@ def test_new_entity(client):
         folder_id=folder_id,
     )
     entity_response = client.post(
-        f"/libraries/{library_id}/entities", json=new_entity.model_dump(mode="json")
+        f"/api/libraries/{library_id}/entities", json=new_entity.model_dump(mode="json")
     )
 
     # Check that the response is successful
@@ -181,7 +249,7 @@ def test_new_entity(client):
 
     # Test for library not found
     invalid_entity_response = client.post(
-        "/libraries/9999/entities", json=new_entity.model_dump(mode="json")
+        "/api/libraries/9999/entities", json=new_entity.model_dump(mode="json")
     )
     assert invalid_entity_response.status_code == 404
     assert invalid_entity_response.json() == {"detail": "Library not found"}
@@ -198,7 +266,7 @@ def test_update_entity(client):
         file_type_group="text",
     )
     update_response = client.put(
-        f"/entities/{entity_id}",
+        f"/api/entities/{entity_id}",
         json=updated_entity.model_dump(mode="json"),
     )
 
@@ -216,7 +284,7 @@ def test_update_entity(client):
 
     # Test for entity not found
     invalid_update_response = client.put(
-        f"/entities/9999",
+        f"/api/entities/9999",
         json=updated_entity.model_dump(mode="json"),
     )
     assert invalid_update_response.status_code == 404
@@ -226,9 +294,16 @@ def test_update_entity(client):
 # Test for getting an entity by filepath
 def test_get_entity_by_filepath(client):
     # Setup data: Create a new library and entity
-    new_library = NewLibraryParam(name="Library for Get Entity Test", folders=["/tmp"])
+    new_library = NewLibraryParam(
+        name="Library for Get Entity Test",
+        folders=[
+            NewFolderParam(
+                path="/tmp", last_modified_at=datetime.now(), type=FolderType.DEFAULT
+            )
+        ],
+    )
     library_response = client.post(
-        "/libraries", json=new_library.model_dump(mode="json")
+        "/api/libraries", json=new_library.model_dump(mode="json")
     )
     library_id = library_response.json()["id"]
 
@@ -243,12 +318,12 @@ def test_get_entity_by_filepath(client):
         folder_id=1,
     )
     entity_response = client.post(
-        f"/libraries/{library_id}/entities", json=new_entity.model_dump(mode="json")
+        f"/api/libraries/{library_id}/entities", json=new_entity.model_dump(mode="json")
     )
     entity_id = entity_response.json()["id"]
 
     get_response = client.get(
-        f"/libraries/{library_id}/entities/by-filepath",
+        f"/api/libraries/{library_id}/entities/by-filepath",
         params={"filepath": new_entity.filepath},
     )
 
@@ -266,7 +341,7 @@ def test_get_entity_by_filepath(client):
 
     # Test for entity not found
     invalid_get_response = client.get(
-        f"/libraries/{library_id}/entities/by-filepath",
+        f"/api/libraries/{library_id}/entities/by-filepath",
         params={"filepath": "nonexistent.txt"},
     )
     assert invalid_get_response.status_code == 404
@@ -274,7 +349,7 @@ def test_get_entity_by_filepath(client):
 
     # Test for library not found
     invalid_get_response = client.get(
-        f"/libraries/9999/entities/by-filepath",
+        f"/api/libraries/9999/entities/by-filepath",
         params={"filepath": new_entity.filepath},
     )
     assert invalid_get_response.status_code == 404
@@ -285,13 +360,19 @@ def test_list_entities_in_folder(client):
     # Setup data: Create a new library and folder
     new_library = NewLibraryParam(name="Library for List Entities Test", folders=[])
     library_response = client.post(
-        "/libraries", json=new_library.model_dump(mode="json")
+        "/api/libraries", json=new_library.model_dump(mode="json")
     )
     library_id = library_response.json()["id"]
 
-    new_folder = NewFoldersParam(folders=["/tmp"])
+    new_folder = NewFoldersParam(
+        folders=[
+            NewFolderParam(
+                path="/tmp", last_modified_at=datetime.now(), type=FolderType.DEFAULT
+            )
+        ]
+    )
     folder_response = client.post(
-        f"/libraries/{library_id}/folders", json=new_folder.model_dump(mode="json")
+        f"/api/libraries/{library_id}/folders", json=new_folder.model_dump(mode="json")
     )
     folder_id = folder_response.json()["folders"][0]["id"]
 
@@ -307,12 +388,12 @@ def test_list_entities_in_folder(client):
         folder_id=folder_id,
     )
     entity_response = client.post(
-        f"/libraries/{library_id}/entities", json=new_entity.model_dump(mode="json")
+        f"/api/libraries/{library_id}/entities", json=new_entity.model_dump(mode="json")
     )
     entity_id = entity_response.json()["id"]
 
     # List entities in the folder
-    list_response = client.get(f"/libraries/{library_id}/folders/{folder_id}/entities")
+    list_response = client.get(f"/api/libraries/{library_id}/folders/{folder_id}/entities")
 
     # Check that the response is successful
     assert list_response.status_code == 200
@@ -328,14 +409,14 @@ def test_list_entities_in_folder(client):
     assert entities_data[0]["file_type_group"] == new_entity.file_type_group
 
     # Test for folder not found
-    invalid_list_response = client.get(f"/libraries/{library_id}/folders/9999/entities")
+    invalid_list_response = client.get(f"/api/libraries/{library_id}/folders/9999/entities")
     assert invalid_list_response.status_code == 404
     assert invalid_list_response.json() == {
         "detail": "Folder not found in the specified library"
     }
 
     # Test for library not found
-    invalid_list_response = client.get(f"/libraries/9999/folders/{folder_id}/entities")
+    invalid_list_response = client.get(f"/api/libraries/9999/folders/{folder_id}/entities")
     assert invalid_list_response.status_code == 404
     assert invalid_list_response.json() == {"detail": "Library not found"}
 
@@ -343,17 +424,47 @@ def test_list_entities_in_folder(client):
 def test_remove_entity(client):
     library_id, _, entity_id = setup_library_with_entity(client)
 
+    # Verify the entity data was automatically inserted into fts and vec tables by event listeners
+    with test_engine.connect() as conn:
+        fts_count = conn.execute(
+            text("SELECT COUNT(*) FROM entities_fts WHERE id = :id"),
+            {"id": entity_id}
+        ).scalar()
+        assert fts_count == 1, "Entity was not automatically added to entities_fts table"
+
+        vec_count = conn.execute(
+            text("SELECT COUNT(*) FROM entities_vec_v2 WHERE rowid = :id"),
+            {"id": entity_id}
+        ).scalar()
+        assert vec_count == 1, "Entity was not automatically added to entities_vec_v2 table"
+
     # Delete the entity
-    delete_response = client.delete(f"/libraries/{library_id}/entities/{entity_id}")
+    delete_response = client.delete(f"/api/libraries/{library_id}/entities/{entity_id}")
     assert delete_response.status_code == 204
 
-    # Verify the entity is deleted
-    get_response = client.get(f"/libraries/{library_id}/entities/{entity_id}")
+    # Verify the entity is deleted from the main table
+    get_response = client.get(f"/api/libraries/{library_id}/entities/{entity_id}")
     assert get_response.status_code == 404
     assert get_response.json() == {"detail": "Entity not found"}
 
+    # Verify the entity is deleted from entities_fts and entities_vec_v2 tables
+    with test_engine.connect() as conn:
+        # Check entities_fts
+        fts_count = conn.execute(
+            text("SELECT COUNT(*) FROM entities_fts WHERE id = :id"),
+            {"id": entity_id}
+        ).scalar()
+        assert fts_count == 0, "Entity was not deleted from entities_fts table"
+
+        # Check entities_vec_v2
+        vec_count = conn.execute(
+            text("SELECT COUNT(*) FROM entities_vec_v2 WHERE rowid = :id"),
+            {"id": entity_id}
+        ).scalar()
+        assert vec_count == 0, "Entity was not deleted from entities_vec_v2 table"
+
     # Test for entity not found in the specified library
-    invalid_delete_response = client.delete(f"/libraries/{library_id}/entities/9999")
+    invalid_delete_response = client.delete(f"/api/libraries/{library_id}/entities/9999")
     assert invalid_delete_response.status_code == 404
     assert invalid_delete_response.json() == {
         "detail": "Entity not found in the specified library"
@@ -369,14 +480,22 @@ def test_add_folder_to_library(client):
     # Create a new library
     new_library = NewLibraryParam(name="Test Library", folders=[])
     library_response = client.post(
-        "/libraries", json=new_library.model_dump(mode="json")
+        "/api/libraries", json=new_library.model_dump(mode="json")
     )
     library_id = library_response.json()["id"]
 
     # Add a new folder to the library
-    new_folders = NewFoldersParam(folders=[tmp_folder_path])
+    new_folders = NewFoldersParam(
+        folders=[
+            NewFolderParam(
+                path=tmp_folder_path,
+                last_modified_at=datetime.now(),
+                type=FolderType.DEFAULT,
+            )
+        ]
+    )
     folder_response = client.post(
-        f"/libraries/{library_id}/folders", json=new_folders.model_dump(mode="json")
+        f"/api/libraries/{library_id}/folders", json=new_folders.model_dump(mode="json")
     )
     assert folder_response.status_code == 200
     assert any(
@@ -385,7 +504,7 @@ def test_add_folder_to_library(client):
     )
 
     # Verify the folder is added
-    library_response = client.get(f"/libraries/{library_id}")
+    library_response = client.get(f"/api/libraries/{library_id}")
     assert library_response.status_code == 200
     library_data = library_response.json()
     folder_paths = [folder["path"] for folder in library_data["folders"]]
@@ -393,7 +512,7 @@ def test_add_folder_to_library(client):
 
     # Test for adding a folder that already exists
     duplicate_folder_response = client.post(
-        f"/libraries/{library_id}/folders", json=new_folders.model_dump(mode="json")
+        f"/api/libraries/{library_id}/folders", json=new_folders.model_dump(mode="json")
     )
     assert duplicate_folder_response.status_code == 400
     assert duplicate_folder_response.json() == {
@@ -402,7 +521,7 @@ def test_add_folder_to_library(client):
 
     # Test for adding a folder to a non-existent library
     invalid_folder_response = client.post(
-        f"/libraries/9999/folders", json=new_folders.model_dump(mode="json")
+        f"/api/libraries/9999/folders", json=new_folders.model_dump(mode="json")
     )
     assert invalid_folder_response.status_code == 404
     assert invalid_folder_response.json() == {"detail": "Library not found"}
@@ -416,7 +535,7 @@ def test_new_plugin(client):
     )
 
     # Make a POST request to the /plugins endpoint
-    response = client.post("/plugins", json=new_plugin.model_dump(mode="json"))
+    response = client.post("/api/plugins", json=new_plugin.model_dump(mode="json"))
 
     # Check that the response is successful
     assert response.status_code == 200
@@ -429,7 +548,7 @@ def test_new_plugin(client):
 
     # Test for duplicate plugin name
     duplicate_response = client.post(
-        "/plugins", json=new_plugin.model_dump(mode="json")
+        "/api/plugins", json=new_plugin.model_dump(mode="json")
     )
     # Check that the response indicates a failure due to duplicate name
     assert duplicate_response.status_code == 400
@@ -439,7 +558,7 @@ def test_new_plugin(client):
 
     # Test for another duplicate plugin name
     another_duplicate_response = client.post(
-        "/plugins", json=new_plugin.model_dump(mode="json")
+        "/api/plugins", json=new_plugin.model_dump(mode="json")
     )
     # Check that the response indicates a failure due to duplicate name
     assert another_duplicate_response.status_code == 400
@@ -456,7 +575,7 @@ def test_update_entity_with_tags(client):
 
     # Make a PUT request to the /libraries/{library_id}/entities/{entity_id} endpoint
     update_response = client.put(
-        f"/entities/{entity_id}",
+        f"/api/entities/{entity_id}",
         json=update_entity_param.model_dump(mode="json"),
     )
 
@@ -480,7 +599,7 @@ def test_patch_tags_to_entity(client):
 
     # Make a PUT request to add initial tags
     initial_update_response = client.put(
-        f"/entities/{entity_id}/tags",
+        f"/api/entities/{entity_id}/tags",
         json=update_entity_param.model_dump(mode="json"),
     )
 
@@ -498,7 +617,7 @@ def test_patch_tags_to_entity(client):
 
     # Make a PATCH request to add new tags
     patch_response = client.patch(
-        f"/entities/{entity_id}/tags",
+        f"/api/entities/{entity_id}/tags",
         json=patch_entity_param.model_dump(mode="json"),
     )
 
@@ -514,7 +633,7 @@ def test_patch_tags_to_entity(client):
     )
 
     # Verify that the tags were actually added by making a GET request
-    get_response = client.get(f"/libraries/{library_id}/entities/{entity_id}")
+    get_response = client.get(f"/api/libraries/{library_id}/entities/{entity_id}")
     assert get_response.status_code == 200
     get_entity_data = get_response.json()
     assert "tags" in get_entity_data
@@ -538,7 +657,7 @@ def test_add_metadata_entry_to_entity_success(client):
 
     # Make a PUT request to the /libraries/{library_id}/entities/{entity_id} endpoint
     update_response = client.put(
-        f"/entities/{entity_id}",
+        f"/api/entities/{entity_id}",
         json=update_entity_param.model_dump(mode="json"),
     )
 
@@ -564,7 +683,7 @@ def test_update_entity_tags(client):
 
     # Make a PUT request to the /libraries/{library_id}/entities/{entity_id} endpoint
     update_response = client.put(
-        f"/entities/{entity_id}",
+        f"/api/entities/{entity_id}",
         json=update_entity_param.model_dump(mode="json"),
     )
 
@@ -573,10 +692,10 @@ def test_update_entity_tags(client):
 
     # Check the response data
     updated_entity_data = update_response.json()
+    print("\nResponse data:", json.dumps(updated_entity_data, indent=2))
     assert "tags" in updated_entity_data
-    assert sorted([t["name"] for t in updated_entity_data["tags"]]) == sorted(
-        tags, key=str
-    )
+    received_tags = sorted([t["name"] for t in updated_entity_data["tags"]])
+    assert received_tags == sorted(tags)
 
 
 def test_patch_entity_metadata_entries(client):
@@ -605,7 +724,7 @@ def test_patch_entity_metadata_entries(client):
 
     # Make a PUT request to the /libraries/{library_id}/entities/{entity_id} endpoint
     patch_response = client.put(
-        f"/entities/{entity_id}",
+        f"/api/entities/{entity_id}",
         json=update_entity_param.model_dump(mode="json"),
     )
 
@@ -615,6 +734,17 @@ def test_patch_entity_metadata_entries(client):
     # Check the response data
     patched_entity_data = patch_response.json()
     expected_data = load_fixture("patch_entity_metadata_response.json")
+
+    # 检查并移除 last_scan_at
+    assert "last_scan_at" in patched_entity_data
+    assert isinstance(patched_entity_data["last_scan_at"], str)
+
+    datetime.fromisoformat(patched_entity_data["last_scan_at"].replace("Z", "+00:00"))
+
+    del patched_entity_data["last_scan_at"]
+    if "last_scan_at" in expected_data:
+        del expected_data["last_scan_at"]
+
     assert patched_entity_data == expected_data
 
     # Update the "author" attribute of the entity
@@ -634,7 +764,7 @@ def test_patch_entity_metadata_entries(client):
 
     # Make a PATCH request to the /libraries/{library_id}/entities/{entity_id}/metadata endpoint
     update_response = client.patch(
-        f"/entities/{entity_id}/metadata",
+        f"/api/entities/{entity_id}/metadata",
         json=update_entity_param.model_dump(mode="json"),
     )
 
@@ -666,7 +796,7 @@ def test_patch_entity_metadata_entries(client):
 
     # Make a PATCH request to the /libraries/{library_id}/entities/{entity_id}/metadata endpoint
     update_response = client.patch(
-        f"/entities/{entity_id}/metadata",
+        f"/api/entities/{entity_id}/metadata",
         json=update_entity_param.model_dump(mode="json"),
     )
 
